@@ -9,7 +9,7 @@ protoc_path = Path(find_executable("protoc") or os.environ.get('PROTOC'))
 
 def test_compile_large_proto():
     module_name = "place"
-    proto = ProtoModule(file_path=f"com/here/platform/opensearch/schema/v2/{module_name}.proto", content="""
+    proto = ProtoModule(file_path=f"com/here/platform/opensearch/schema/v2/{module_name}.proto", source="""
 // This file is part of the OpenSearch API.
 // Copyright (c) HERE 2019. All rights reserved.
 //
